from textual import on
from textual.app import ComposeResult
from textual.screen import ModalScreen
from textual.widgets import Select


class ThemeSelect(ModalScreen):
    """Screen with themes selector."""

    def compose(self) -> ComposeResult:
        yield Select.from_values(
            [
                "[#F8DE7E]vegas",
                "[#0BDA51]malachite",
                "[#B026FF]neon",
                "[#FF5733]crimson",
                "[#0000ff]blue",
            ],
            prompt="Select a theme",
            id="themes_content",
        )

    @on(Select.Changed)
    def select_changed(self, event: Select.Changed) -> None:
        selected_theme = event.value
        if selected_theme == Select.BLANK:
            return
        theme_name = str(event.value).split("]")[-1]
        self.dismiss(theme_name)



from textual.app import Widget, ComposeResult
from textual.widgets import Footer


class TableFooter(Widget):

    def compose(self) -> ComposeResult:
        yield Footer()

    def on_mount(self) -> None:
        self.styles.height = "1"



import sys
import os

from pqviewer.file_parser import FileParser
from pqviewer.app import PQViewer
from pqviewer.exceptions import pretty_print_error


def pqviewer() -> None:
    """Main entry point"""

    file_path = sys.argv[1]
    if os.path.exists(file_path):
        try:
            parsed_file = FileParser(file_path)
        except Exception as e:
            sys.exit(pretty_print_error(e))
        app = PQViewer(parsed_file)
        app.run()
    else:
        sys.exit(
            pretty_print_error(f"The specified path does not exist.\n'{file_path}'")
        )



from textual.app import Widget, ComposeResult
from textual.widgets import DataTable

from pqviewer.file_parser import FileParser


class TableContent(Widget):

    def __init__(self, parsed_file: FileParser) -> None:
        self.parsed_file = parsed_file
        super().__init__()

    def compose(self) -> ComposeResult:
        yield DataTable(id="datatable_content")

    def on_mount(self) -> None:
        table = self.query_one(DataTable)
        table_content = self.parsed_file.to_table()
        table.add_columns(*table_content[0])
        table.add_rows(table_content[1:])



from rich import print
from rich.panel import Panel


def pretty_print_error(error: Exception) -> None:
    return print(
        Panel.fit(
            str(error),
            title="PQViewer encountered an error.",
            title_align="left",
            border_style="red",
            highlight=True,
        )
    )



from textual.app import ComposeResult
from textual.containers import Vertical
from textual.screen import ModalScreen
from textual.widgets import Static


class HelpScreen(ModalScreen):
    """Screen with PQViewer bindings."""

    keybinds = """Arrows: Move around the table\nPgUp:   Scroll up\nPgDn:   Scroll down\nTab:    Switch focus"""

    def compose(self) -> ComposeResult:
        with Vertical(id="help_panel"):
            yield Static(self.keybinds, id="help_content")
            yield Static("Press any key to return to table.", id="help_footer")

    def on_mount(self) -> None:
        help_panel = self.query_one("#help_panel")
        help_panel.border_title = "PQViewer key binds"

    def on_key(self) -> None:
        self.app.pop_screen()



from typing import List

import pyarrow.parquet as pq


class FileParser:
    def __init__(self, file_name: str) -> None:
        self.file_name = file_name
        self.table = pq.read_table(self.file_name)

    def schema(self) -> List[tuple]:
        """Transform table schema to list."""

        return [
            (name, str(self.table.schema.field(name).type))
            for name in self.table.column_names
        ]

    def number_columns(self) -> int:
        """Get number of columns."""
        return self.table.num_columns
    
    def number_rows(self) -> int:
        """Get number of rows."""
        return self.table.num_rows

    def to_table(self) -> List[tuple]:
        """Transform table content to list."""

        table_header = [tuple(self.table.column_names)]
        table_content = [tuple(row.values()) for row in self.table.to_pylist()]
        return table_header + table_content




from textual.app import Widget, ComposeResult
from textual.widgets import Label

from pqviewer.file_parser import FileParser


class TableHeader(Widget):

    def __init__(self, parsed_file: FileParser, id: str) -> None:
        self.file_name = parsed_file.file_name
        super().__init__(id=id)

    def compose(self) -> ComposeResult:
        yield Label(self.file_name, shrink=True)

    def on_mount(self) -> None:
        self.styles.height = "auto"
        self.border_title = "File"
        self.styles.color = "#777"



from textual.app import App, ComposeResult
from textual.containers import Horizontal
from textual.binding import Binding
from textual.reactive import var

from pqviewer.table_header import TableHeader
from pqviewer.table_schema import TableSchema
from pqviewer.table_content import TableContent
from pqviewer.table_footer import TableFooter
from pqviewer.help_screen import HelpScreen
from pqviewer.themes_screen import ThemeSelect


THEMES = {
    "vegas": "#F8DE7E",
    "malachite": "#0BDA51",
    "neon": "#B026FF",
    "crimson": "#FF5733",
    "blue": "#0000ff",
}


class PQViewer(App):

    CSS_PATH = "global.tcss"
    BINDINGS = [
        Binding("ctrl+c", "quit", "Quit"),
        Binding("h", "help_screen", "Help"),
        Binding("t", "themes_screen", "Themes"),
    ]
    theme = var("vegas")

    def __init__(self, parsed_file) -> None:
        self.parsed_file = parsed_file
        super().__init__()

    def compose(self) -> ComposeResult:
        yield TableHeader(self.parsed_file, id="table_header")
        with Horizontal():
            yield TableSchema(self.parsed_file)
            yield TableContent(self.parsed_file)
        yield TableFooter()

    def watch_theme(self):
        color_hex = THEMES[self.theme]
        table_header = self.query_one("#table_header")
        table_header.styles.border = ("heavy", color_hex)

        datatable_schema = self.query_one("#datatable_schema")
        datatable_schema.styles.border = ("heavy", color_hex)
        datatable_schema.styles.scrollbar_color = color_hex
        datatable_schema.styles.scrollbar_color_active = color_hex
        datatable_schema.styles.scrollbar_color_hover = color_hex
        datatable_schema.border_title = "Schema"
        datatable_schema.border_subtitle = f"Total columns: {self.parsed_file.number_columns()}"

        datatable_content = self.query_one("#datatable_content")
        datatable_content.styles.border = ("heavy", color_hex)
        datatable_content.styles.scrollbar_color = color_hex
        datatable_content.styles.scrollbar_color_active = color_hex
        datatable_content.styles.scrollbar_color_hover = color_hex
        datatable_content.border_title = "Table"
        datatable_content.border_subtitle = f"Total rows: {self.parsed_file.number_rows()}"

    def action_help_screen(self) -> None:
        """Action to display the help screen dialog."""
        self.push_screen(HelpScreen())

    def action_themes_screen(self):
        self.push_screen(ThemeSelect(), callback=self.modal_callback_to_exit)

    def modal_callback_to_exit(self, selected_theme):
        self.theme = selected_theme
